//
// JTAppUI.cpp
//

// JTelnet's application UI class.

#include <coeutils.h>

#include <eikenv.h>
#include <eikcmds.hrh>
#include <eikmenu.hrh>
#include <eiktbar.h>
#include <eikon.rsg>
#include <eikmenub.h>
#include <eikcmbut.h>
#include <eikfnlab.h>

#include "JTAppUI.h"
#include "JTelnet.hrh"
#include <JTelnet.rsg>

#include "Terminals.h"
#include "HelpTask.h"


//
// ConstructL
//
void CJTAppUi::ConstructL() 
{
    // Allow base class (CEikAppUi) to perform necessary construction
    BaseConstructL();

    // query the network now, so we dont do it when the menu is first displayed
    JTApp()->IsNetworkOpen();

    // By default, toolbar is on
    iShowToolBar = ETrue;

    iAppZoom.SetGraphicsDeviceMap(iCoeEnv->ScreenDevice());
    iAppZoom.SetZoomFactor(MTerminal::EZoomStateTwo);

    // Construct the container control which forms the main view
    // for this application. Construction requires the rectangle
    // available and an observer for phase events.
    iContainer = CTTY::NewLC(ClientRect(), this);

    Terminal()->SetZoomFactorL(&iAppZoom);

    iHelp = CHelpTask::NewLC(Application()->AppFullName());

    // The container is added to the control stack (for key event
    // handling).
    AddToStackL(iContainer);

    ToolButtonSetDimmed(EJTelnetDisconnectOrCancel, EDimmed);

    iExitingAsync = EFalse;

    CleanupStack::Pop(2);
}
	

//
// Destructor
//
CJTAppUi::~CJTAppUi() 
{
    delete iHelp;
    delete iContainer;
}


//
// ProcessCommandParametersL
//
// simply delegate to the base class...
TBool CJTAppUi::ProcessCommandParametersL(TApaCommand aCommand,TFileName& aDocumentName,const TDesC& /*aTail*/) 
{
    return CEikAppUi::ProcessCommandParametersL(aCommand, aDocumentName);
}


//
// HandleCommandL
//
// Direct user commands to appropriate handlers.
void CJTAppUi::HandleCommandL(TInt aCommand) 
{
    switch (aCommand) {
    // Create new file command
    case EEikCmdFileNew:			// <---Pre-defined command id
	OnCmdFileNewL();      
	break;
    
    // Open File command
    case EEikCmdFileOpen:           // <---Pre-defined command id
	OnCmdFileOpenL();
	break;

    // Save in a different file command
    case EEikCmdFileSaveAs:         // <---Pre-defined command id
	OnCmdFileSaveAsL();
	break;

    case EEikCmdExit:
	OnCmdExit();
	break;

    case EEikCmdEditCopy:           // <---Pre-defined command id
	Terminal()->ClipboardL(MTerminal::ECopy);
	break;

    case EEikCmdEditPaste:
//	Terminal()->ClipboardL(MTerminal::EPaste);
	break;
    
    case EEikCmdEditSelectAll:
	Terminal()->SelectAllL();
	break;

    case EJTelnetCmdShowToolbar:
	OnCmdToggleShowToolbar();
	break;

    //-------------------------------------------------------------------

#if 0
    case EEikCmdZoomIn:
	OnCmdZoomIn();
	break;

    case EEikCmdZoomOut:
	OnCmdZoomOut();
	break;

	// Infrared send command
    case EEikCmdIrdaSend:
	iEikonEnv->InfoMsg(R_JTELNET_TEXT_IRDASEND);
	break;
	
	// Infrared receive command
    case EEikCmdIrdaReceive:
	iEikonEnv->InfoMsg(R_JTELNET_TEXT_IRDARECEIVE);
	break;
#endif

    case EEikCmdZoomIn:
    case EEikCmdZoomOut:
	iEikonEnv->InfoMsg(R_EIK_TBUF_NOT_AVAILABLE);
	break;

    case EEikCmdHelpContents:
	iHelp->DisplayL();
	break;

    case EEikCmdHelpAbout:
	OnCmdAboutL();
	break;

    // ------------
    case EJTelnetConnect:
	OnCmdConnectL();
	break;

    case EJTelnetDisconnectOrCancel:
	Terminal()->DisconnectL(MTerminal::EDisconnectNormal);
	break;

    case EJTelnetServerDetails:
	OnCmdServerDetailsL();
	break;

    case EJTelnetDisconnectNetwork:
	OnCmdDisconnectNetL();
	break;

    default:
	break;
    }
}


//
// DynInitMenuPaneL
//
// update the view state of menu items
void CJTAppUi::DynInitMenuPaneL(TInt aMenuId,CEikMenuPane* aMenuPane) 
{
    switch(aMenuId) 
    {
    case R_JTELNET_VIEW_MENU:
	aMenuPane->SetItemButtonState(EJTelnetCmdShowToolbar, iShowToolBar ? EEikMenuItemSymbolOn : 0);
	break;

    case R_EIK_SIDEBAR_EDIT_MENU:
	aMenuPane->SetItemDimmed(EEikCmdEditCut, ETrue);
	aMenuPane->SetItemDimmed(EEikCmdEditPaste, ETrue);
	break;

    case R_EIK_SIDEBAR_IRDA_MENU:
	aMenuPane->SetItemDimmed(EEikCmdIrdaSend, ETrue);
	aMenuPane->SetItemDimmed(EEikCmdIrdaReceive, ETrue);
	break;

    case R_JTELNET_FILE_MENU:
	aMenuPane->SetItemDimmed(EJTelnetDisconnectNetwork, !JTApp()->IsNetworkOpen());
	break;

    case R_JTELNET_SESSION_MENU:
	aMenuPane->SetItemDimmed( EJTelnetConnect
	                        ,   !JTDoc()->CanConnectL()
				 || Terminal()->IsActive()
				);
	aMenuPane->SetItemDimmed( EJTelnetDisconnectOrCancel
	                        , Terminal()->IsDisconnected()
				);
	break;

    default:
	break;
    }
}


//
// OnCmdAboutL
//
void CJTAppUi::OnCmdAboutL() 
{
    CEikDialog* dialog = new (ELeave) CEikDialog();
    dialog->ExecuteLD(R_JTELNET_ABOUT_DIALOG);
}


//
// OnCmdDisconnectNetL
//
void CJTAppUi::OnCmdDisconnectNetL() {
    CEikDialog* dialog = new (ELeave) CEikDialog();
    
    if (dialog->ExecuteLD(R_JTELNET_END_NET_DIALOG) == EJTelnetBidDisconnect) 
    {
	if (Terminal()->IsDisconnected()) 
	{
	    JTApp()->CloseNetwork();
	}
	else 
	{
	    // disconnect the NVT...
	    Terminal()->DisconnectL(MTerminal::EDisconnectNet);
	}
    }
}


//
// OpenFileL
//
void CJTAppUi::OpenFileL(const TDesC& aFileName)
{
    SaveAnyChangesL();
    JTDoc()->DoOpenFileL(aFileName);
    HandleModelChangeL();
}


//
// CreateFileL
//
void CJTAppUi::CreateFileL(const TDesC& aFileName)
{
    SaveAnyChangesL();
    CUIInetAddr* pDummy = CUIInetAddr::NewLC(_L(""), MTerminal::EDefaultPort);
    JTDoc()->DoNewFileL(aFileName, *pDummy);
    HandleModelChangeL();
    CleanupStack::Pop();
}


//
// OnCmdExit
//
// if we are connected, do an async, orderly, shutdown.
void CJTAppUi::OnCmdExit() 
{
    // Save any changes before exiting from the app
    SaveAnyChangesL();

    if (!Terminal()->IsDisconnected()) 
    {
	Terminal()->DisconnectL(MTerminal::EDisconnectNormal);
	iExitingAsync = ETrue;
    }

    iHelp->Close();

    if (!iExitingAsync) 
    {
	Exit();
    }
}


#if 0
// This zoom code doesnt work...

void CJTAppUi::OnCmdZoomIn() {

    TInt current = iAppZoom.ZoomFactor();

    switch (current) {
    case MTerminal::EZoomStateOne:
	current = MTerminal::EZoomStateTwo;
	break;
    case MTerminal::EZoomStateTwo:
	current = MTerminal::KZoomStateThree;
	break;
    case MTerminal::KZoomStateThree:
	current = MTerminal::KZoomStateFour;
	break;
    case MTerminal::KZoomStateFour:
	current = MTerminal::EZoomStateOne;
	break;
    }
    iAppZoom.SetZoomFactor(current);
    Terminal()->SetZoomLevelL(MTerminal::TZoomLevel(current));
}


void CJTAppUi::OnCmdZoomOut() {
    
    TInt current = iAppZoom.ZoomFactor();

    switch (current) {
    case MTerminal::EZoomStateOne:
	current = MTerminal::KZoomStateFour;
	break;
    case MTerminal::EZoomStateTwo:
	current = MTerminal::EZoomStateOne;
	break;
    case MTerminal::KZoomStateThree:
	current = MTerminal::EZoomStateTwo;
	break;
    case MTerminal::KZoomStateFour:
	current = MTerminal::KZoomStateThree;
	break;
    }
    iAppZoom.SetZoomFactor(current);
    Terminal()->SetZoomLevelL(MTerminal::TZoomLevel(current));
}
#endif


//
// OnCmdToggleShowToolbar
//
void CJTAppUi::OnCmdToggleShowToolbar()
{
    iShowToolBar= !iShowToolBar;
    DrawViewL();
}

	
//
// OnCmdFileSaveL
//
// Note that in JTelnet this is never triggered by the user...
void CJTAppUi::OnCmdFileSaveL() 
{
    // Save
    SaveL();

    HandleModelChangeL();

    // Issue an info message confirmation if not running as embedded.
    if (!iContainerAppUi) 
    {
	iEikonEnv->InfoMsg(R_JTELNET_TEXT_SAVED);
    }
}


//
// OnCmdConnectL
//
// If a server address is available initiate a connection.
// If no address is entered yet, prompt the user for one...
void CJTAppUi::OnCmdConnectL() 
{
    if (!JTDoc()->CanConnectL()) 
    {
	OnCmdServerDetailsL();
    }

    if (   JTDoc()->CanConnectL()
	&& Terminal()->IsDisconnected()
	) 
    {
	JTApp()->DisableRemoteLink();

	Terminal()->ConnectL( JTDoc()->GetServerAddress().Name()
			    , JTDoc()->GetServerAddress().Port()
			    );
    }
}


//
// NotifyPhaseChange
//
void CJTAppUi::NotifyPhaseChange( MTerminal::EConnectPhase prev
			        , MTerminal::EConnectPhase current
				) 
{
    switch (current) 
    {
    case MTerminal::ETermDisconnected:
	if (iExitingAsync) 
	{   
	    // complete an async shutdown
	    iExitingAsync = EFalse;
	    JTApp()->ReEnableRemoteLink();
	    Exit();
	    return;
	}

	ToolButtonSetTextL(EJTelnetDisconnectOrCancel, R_JTELNET_DISCONNECT);
	ToolButtonSetDimmed(EJTelnetConnect, ENormal);
	ToolButtonSetDimmed(EJTelnetDisconnectOrCancel, EDimmed);
	if (prev == MTerminal::ETermConnected) {
	    iEikonEnv->InfoMsg(R_JTELNET_REMOTE_DISCONNECTED);
	}
	else if (prev == MTerminal::ETermDisconnecting) {
	    iEikonEnv->InfoMsg(R_JTELNET_DISCONNECTED);
	}
	else if (prev == MTerminal::ETermConnecting) {
	    iEikonEnv->InfoMsg(R_JTELNET_NO_CONNECTION);
	}
	break;
    case MTerminal::ETermConnecting:
	ToolButtonSetTextL(EJTelnetDisconnectOrCancel, R_JTELNET_CANCEL_CONNECT);
	ToolButtonSetDimmed(EJTelnetConnect, EDimmed);
	ToolButtonSetDimmed(EJTelnetDisconnectOrCancel, ENormal);
	break;
    case MTerminal::ETermConnected:
	ToolButtonSetTextL(EJTelnetDisconnectOrCancel, R_JTELNET_DISCONNECT);
	ToolButtonSetDimmed(EJTelnetConnect, EDimmed);
	ToolButtonSetDimmed(EJTelnetDisconnectOrCancel, ENormal);
	iEikonEnv->InfoMsg(R_JTELNET_TEXT_CONNECTED);
	break;
    case MTerminal::ETermDisconnecting:
	ToolButtonSetTextL(EJTelnetDisconnectOrCancel, R_JTELNET_DISCONNECT);
	ToolButtonSetDimmed(EJTelnetConnect, EDimmed);
	ToolButtonSetDimmed(EJTelnetDisconnectOrCancel, ENormal);
	break;
    }
}


//
// OnAddressResolved
//
void CJTAppUi::OnAddressResolved(TRequestStatus aStatus) 
{
    if (aStatus == KErrNone) 
    {
        iEikonEnv->InfoMsg(R_JTELNET_RESOLVED_ADDR);
    }
    else 
    {
        iEikonEnv->InfoMsg(R_JTELNET_NO_ADDRESS);
    }
}


// unused MTerminalObserver protcol placeholders
void CJTAppUi::OnServerStart(TRequestStatus) {
}
void CJTAppUi::OnConnect(TRequestStatus) {
}
void CJTAppUi::OnDisconnect(TRequestStatus) {
}


//
// ToolButtonSetTextL
//
void CJTAppUi::ToolButtonSetTextL(TInt aId, TInt aResourceId) 
{
    CEikCommandButton* button = STATIC_CAST(CEikCommandButton*, iToolBar->ControlById(aId));

    HBufC* buffer = iEikonEnv->AllocReadResourceLC(aResourceId);
    button->SetTextL(buffer->Des());
    button->DrawNow();

    CleanupStack::PopAndDestroy();
}


//
// ToolButtonSetDimmed
//
void CJTAppUi::ToolButtonSetDimmed(TInt aId, EBtnVisibility aDimmed) 
{
    CEikCommandButton* button = STATIC_CAST(CEikCommandButton*, iToolBar->ControlById(aId));

    button->SetDimmed(aDimmed == EDimmed);
    button->DrawNow();

}


//
// OnCmdServerDetailsL
//
void CJTAppUi::OnCmdServerDetailsL() 
{
    TBool read_only = Terminal()->IsConnected();
    CJTServerDetailsDlg* dialog = new (ELeave) CJTServerDetailsDlg( JTDoc()->GetServerAddress()
								  , read_only
								  );
    if (   dialog->ExecuteLD(R_JTELNET_SERVER_DETAILS_DIALOG)
	&& !read_only
	) 
    {
	OnCmdFileSaveL();
    }
}


//
// OnCmdFileNewL
//
void CJTAppUi::OnCmdFileNewL() 
{
    // Save any changes to the current file
    SaveAnyChangesL();
    // Fetch drive and path of current main document file and
    // ensure that it exists.
    TParsePtrC  parser(Document()->Process()->MainDocFileName());
    TFileName   fileName = parser.DriveAndPath();
    ConeUtils::EnsurePathExistsL(fileName);
    // Construct the dialog requesting the name of new document file.
    
    CUIInetAddr* name = CUIInetAddr::NewLC(_L(""), MTerminal::EDefaultPort);
    
    CJTNewFile* dialog   = new (ELeave) CJTNewFile( &fileName
						  , NULL
						  , R_JTELNET_CREATE_NEW_FILE_DIALOG_TITLE
						  , *name
						  );
    
    // Initiate the dialog; if user enters a name and presses OK, then:
    // 
    // 1. Call document's DoNewFileL() to create a new document file store
    // 2. Inform the user interface that the document has changed
    // 3. Change window group name and "last used" file name in "ini" file
    if (dialog->ExecuteLD(R_EIK_DIALOG_FILE_SAVEAS))
    {
	JTDoc()->DoNewFileL(fileName, *name);
	HandleModelChangeL();
    }
    
    CleanupStack::PopAndDestroy();
    
}


//
// OnCmdFileOpenL
//
void CJTAppUi::OnCmdFileOpenL()
{
    // Save any changes to the current file
    SaveAnyChangesL();
    // Fetch drive and path of current main document file and
    // ensure that it exists.
    TParsePtrC  parser(Document()->Process()->MainDocFileName());
    TFileName   fileName = parser.DriveAndPath();
    ConeUtils::EnsurePathExistsL(fileName);
    // Construct the dialog requesting the name of the document file.
    CEikFileOpenDialog* dialog=new(ELeave) CEikFileOpenDialog(&fileName);
    // Restrict the files listed to those created by this application
    dialog->RestrictToNativeDocumentFiles();
    // Initiate the dialog; if user enters a name and presses OK, then:
    //
    // 1. Call document's DoOpenFileL() to open the document file store
    // 2. Inform the user interface that the document has changed
    // 3. Change window group name and "last used" file name in "ini" file
    if (dialog->ExecuteLD(R_EIK_DIALOG_FILE_OPEN))
    {
	JTDoc()->DoOpenFileL(fileName);
	HandleModelChangeL();
    }
}


//
// OnCmdFileSaveAsL
//
void CJTAppUi::OnCmdFileSaveAsL()
{
    // Fetch drive and path of current main document file and
    // ensure that it exists.
    TParsePtrC  parser(Document()->Process()->MainDocFileName());
    TFileName   fileName = parser.DriveAndPath();
    ConeUtils::EnsurePathExistsL(fileName);
    // Construct the dialog requesting the name that the document file
    // is to be saved as.
    CEikDialog* dialog=new(ELeave) CEikFileSaveAsDialog(&fileName,NULL,NULL);
    // Initiate the dialog; if user enters a name and presses OK, then:
    //
    // 1. Save the document if the name is the same as the current one
    //    and return.
    // 2. Delete any existing file of the same name
    // 3. Change window group name and "last used" file name in "ini" file
    if (dialog->ExecuteLD(R_EIK_DIALOG_FILE_SAVEAS))
    {
	if (JTDoc()->FileNameExists(fileName))
	{
	    OnCmdFileSaveL();
	    return;
	}
	TEntry entry;
	if (!iEikonEnv->FsSession().Entry(fileName,entry))
	{
	    User::LeaveIfError(iEikonEnv->FsSession().Delete(fileName));
	}

	JTDoc()->DoSaveToNewFileL(fileName);
	HandleModelChangeL();
	iEikonEnv->InfoMsg(R_JTELNET_TEXT_SAVED);
    }  
}
 

//
// HandleModelChangeL
//
void CJTAppUi::HandleModelChangeL() 
{
    if (!JTDoc()->GetServerAddress().IsValidHostL()) 
    {
	iEikonEnv->InfoMsg(R_JTELNET_CANT_SET_ADDRESS);
    }

    iEikonEnv->UpdateTaskNameL();
    CEikFileNameLabel* fname=STATIC_CAST(CEikFileNameLabel*,iToolBar->ControlById(EJTelnetCmdDocName));
    fname->UpdateL();
    fname->DrawNow();

    ResetViewL();
}


//
// ResetViewL
//
void CJTAppUi::ResetViewL()
{
    DrawViewL();
}


//
// DrawViewL
// 
void CJTAppUi::DrawViewL()
{
    iToolBar->MakeVisible(iShowToolBar);
    iToolBar->ActivateL();	
    iContainer->SetRectL(ClientRect());
    iContainer->DrawNow();
}
