// JTelnet.H
//
// Copyright (c) 1999 John McAleely
//

#if !defined(__JTELNET_H__)
#define __JTELNET_H__

#include <eikcfdlg.h>
#include <eikapp.h>
#include <eikdoc.h>

#include <nifman.h>
#include <s32std.h>

class CEikEdwin;

// The unique identifier for this application.
const TUid KUidJTelnetApp={0x100006B2};

//
// CUIInetAddr
//
// A user entered internet host name.
class CUIInetAddr : public CBase {
    TUint  iPort;
    HBufC* iName;

    CUIInetAddr(const CUIInetAddr&);
    CUIInetAddr& operator=(const CUIInetAddr&);

    CUIInetAddr();
    void ConstructL(const TDesC& aName, const TUint& aPort);
    void ConstructL(const CEikEdwin* aEdwin, const TUint& aPort);
    
public:
    

    ~CUIInetAddr();

    // like a copy constuctor but can leave
    void CopyL(const CUIInetAddr&);

    static CUIInetAddr* NewLC(const TDesC& aName, const TUint& aPort);
    static CUIInetAddr* NewL(const TDesC& aName, const TUint& aPort);

    static CUIInetAddr* NewLC(const CEikEdwin* aEdwin, const TUint& aPort);
    static CUIInetAddr* NewLC(const CStreamStore& aStore, TStreamId anId);

    TStreamId StoreL(CStreamStore& aStore);
    void      RestoreL(const CStreamStore& aStore,TStreamId anId);
    void      ExternalizeL(RWriteStream& aStream) const;
    void      InternalizeL(RReadStream& aStream);


    const TDesC& Name() const { return *iName; }
    const TUint& Port() const { return iPort; }
    void NameL(const TDesC& aName);
    void Port(const TUint& aPort) { iPort = aPort; }

    void GetNameL(const CEikEdwin* aEdwin);	// get the name from this edwin...
    
    TBool IsValidHostL() const;

    static TBool IsValidHostNameL(CEikEdwin* aEdwin);
    static TBool IsValidHostNameL(const TDesC& aName);
};


class CJTNewFile : public CEikFileSaveAsDialog {
public:

    CJTNewFile( TDes*  aFileName
	      , TBool* aUseNewFile
	      , TInt aTitleId
	      , CUIInetAddr& aAddr
	      );
private:

    TBool OkToExitL(TInt aButtonId);
    void PreLayoutDynInitL();

    CUIInetAddr& iAddr;
};

class CJTServerDetailsDlg : public CEikDialog {
public:
    CJTServerDetailsDlg(CUIInetAddr&, TBool aReadOnly);

private:
    TBool OkToExitL(TInt aButtonId);
    void PreLayoutDynInitL();

    CUIInetAddr& iAddr;
    TBool        iReadOnly;
};


//
// CJTDocument
//
// Stores the state needed for each sever connection.
// principally the address and port number.
class CJTDocument : public CEikDocument {

    CJTDocument(CEikApplication& aApp);

public:
    // Construction.
    static CJTDocument* NewL(CEikApplication& aApp);
    // Destruction.
    ~CJTDocument();

private:
    // Defined as pure virtual by CEikDocument; full implementation
    // provided by this class
    CEikAppUi* CreateAppUiL();		// Construct an app.user interface

    // Defined as pure virtual by CApaDocument; empty implementation 
    // provided by CEikDocument; full implementation provided
    // by this class.
    void       NewDocumentL();   	// Build a new document
    void       NewDocumentL(const CUIInetAddr& aAddr);

    // Defined as pure virtual by CApaDocument; empty implementation
    // provided by CEikDocument; full implementation provided
    // by this class.
    void       StoreL( CStreamStore& aStore
	             , CStreamDictionary& aStreamDic
		     ) const;
    void       RestoreL( const CStreamStore& aStore
	               , const CStreamDictionary& aStreamDic
		       );

public:
    // Member functions defined by this class
    void       DoNewFileL(const TFileName& aFileName, const CUIInetAddr& aAddr);
    void       DoOpenFileL(const TFileName& aFileName);
    void       DoSaveToNewFileL(const TFileName& aNewFileName);
    TBool      FileNameExists(const TFileName& aFileName) const;

    // ETrue if connection is possible
    TBool      CanConnectL() const { return iAddr->IsValidHostL(); }

    CUIInetAddr& GetServerAddress() { return *iAddr; }

private:
    // Member functions defined by this class
    void       CreateModelL();
    void       ResetModelL();

private:

    CUIInetAddr* iAddr;
};


//
// CJTelnetApp
//
// the application object.
class CJTelnetApp : public CEikApplication {
public:

    CJTelnetApp();
    ~CJTelnetApp();

    TBool IsNetworkOpen();
    void  CloseNetwork();

    void  DisableRemoteLink();
    void  ReEnableRemoteLink();

    void   SetPrefTInt32L(TInt32 aPref, const TUid& aUid);
    TInt32 GetPrefTInt32L(const TUid& aUid, TInt32 aDefaultPref = 0);

    // gather values for the ini file here, so we can
    // be sure of uniqueness.
    enum IniEntries {
	BufferLimit = 0x01100502
    };

private:

    // Defined as pure virtual by CApaApplication; implementation
    // provided by this class 
    TUid          AppDllUid() const; // Returns Uid associated with app

    // Defined as pure virtual by CEikApplication; implementation
    // provided by this class.
    CApaDocument* CreateDocumentL(); // Construct new document

    RNifMonitor iNif;	    // the network interface layer.
    TBool       iNIFOpened;

    TBool       iReEnableLink;
};

#endif